﻿using System.Collections;
using System.Collections.Generic;
using UnityEngine;

public class transferBehavior : MonoBehaviour
{
    public GameObject left_elevator;
    public GameObject right_elevator;
    public GameObject machineArm;
    public GameObject controller;
    /// <summary>
    /// 0 no cmd, 1 park get car from arm, 2 park put car in space, 3 fetch get car from space, 4 fetch wait for arm to get car
    /// </summary>
    public int cmd;
    public int targetFloor;
    public int targetX;
    public float xSpeed;
    public float zSpeed;
    public float mainZ; // value in z axis st. vehicles can move along x axis
    public float spaceZ; // value in z axis for spaces
    public GameObject load; // whether carrying a car
    public int state; // 0 idle, 1 move to elevator and wait, 2 move with elevator, 3 move to the targetX, 4 put/take car action according to cmd, 5 move out, 6 waiting for next move cmd
    public Vector3 currPos;
    public bool waitForElevator;
    public bool oneDirection;

    private const float maxX = 24f;
    private const float minX = -2f;
    private bool currElevatorLeft;
    private static float[] termPosX = { 1.2f, 6.2f, 8.7f, 13.7f, 16.2f, 21.2f };

    // Start is called before the first frame update
    void Start()
    {
        xSpeed = 6f;
        zSpeed = 6f;
        mainZ = -2.9f;
        spaceZ = -8.9f;
        targetFloor = -1;
        targetX = -1;
        state = 0;
        waitForElevator = false;
        oneDirection = true;
        currPos = GetComponent<Transform>().position;
        currElevatorLeft = true;
        load = null;
    }

    public void SetLoad(GameObject ld)
    {
        load = ld;
    }

    public void SetTarget(int floor, int x)
    {
        targetFloor = floor;
        targetX = x;
    }

    public void SendCmd(int cmd)
    {
        this.cmd = cmd;
    }

    public int GetFloor()
    {
        return Mathf.RoundToInt(transform.position.y / 3.0f + 1);
    }

    public int GetSpace()
    {
        return Mathf.RoundToInt((transform.position.x + 6.3f) * 14.0f / 35.0f) + 1;
    }

    /// <summary>
    ///  获取离中跑车最近终端编号0-5
    /// </summary>
    /// <returns></returns>
    public int GetNearestTerm()
    {
        // 非二层
        if (currPos.y > 5f)
        {
            return -1;
        }
        // 已靠边
        if (currPos.x < -1 || currPos.x > 23)
        {
            return -1;
        }
        else
        {
            float diff = 99f;
            int index = -1;
            for (int i = 0; i < termPosX.Length; i++)
            {
                if(diff > Mathf.Abs(currPos.x - termPosX[i]))
                {
                    diff = Mathf.Abs(currPos.x - termPosX[i]);
                    index = i;
                }
            }
            return index;
        }
    }

    public void TransferMove(int axis, float diff, int toState)
    {
        if (axis == 0)
        {
            if (currPos.y < 5f)//2楼运动检查
            {
                for (int i = termPosX.Length - 1; i >= 0; i--)
                {
                    // 找到其右边第一个终端
                    if (currPos.x > termPosX[i])
                    {
                        // 未上锁等待
                        if (controller.GetComponent<controller>().doorLock[i] != 1)
                        {
                            Debug.Log("二楼中跑车等待锁门");
                            return;
                        }
                    }
                }
            }
            if (Mathf.Abs(diff) < Time.deltaTime * 15)
            {
                state = toState;
            }
            else if (diff < 0)
            {
                transform.Translate(Vector3.right * xSpeed * Time.deltaTime);
            }
            else
            {
                transform.Translate(Vector3.left * xSpeed * Time.deltaTime);
            }
        }
        else if (axis == 2)
        {
            if (Mathf.Abs(diff) < 0.25f)
            {
                state = toState;
            }
            else if (diff < 0)
            {
                transform.Translate(Vector3.forward * zSpeed * Time.deltaTime);
            }
            else
            {
                transform.Translate(Vector3.back * zSpeed * Time.deltaTime);
            }
        }
    }

    // Update is called once per frame
    void Update()
    {
        currPos = GetComponent<Transform>().position;
        if (load != null)
        {
            load.GetComponent<Transform>().position = currPos + Vector3.up * 0.15f;
        }
        switch (state)
        {
            case 0:// 空闲
                if (targetFloor > 0 && targetX > 0)
                {
                    if (Mathf.Abs((targetFloor - 1) * 3.0f - currPos.y) < Time.deltaTime * 15)
                    {
                        state = 3;
                    }
                    else
                    {
                        state = 1;
                    }
                }
                break;
            case 1:// 运动到电梯
                waitForElevator = true;
                float diff = 0f;
                // 已经在左电梯上
                if (currPos.x >= maxX - 0.5f && Mathf.Abs(left_elevator.GetComponent<elevatorBehavior>().GetFloor() - currPos.y) < 0.5f && left_elevator.GetComponent<elevatorBehavior>().state == 0)
                {
                    currElevatorLeft = true;
                    if (currPos.x < 27f)
                    {
                        transform.Translate(Vector3.right * xSpeed * Time.deltaTime);
                    }
                    else
                    {
                        waitForElevator = false;
                        state = 2;
                    }
                }
                // 已经在右电梯上
                else if (currPos.x < minX + 0.5f && Mathf.Abs(right_elevator.GetComponent<elevatorBehavior>().GetFloor() - currPos.y) < 0.5f && left_elevator.GetComponent<elevatorBehavior>().state == 0)
                {
                    currElevatorLeft = false;
                    if (currPos.x > -5f)
                    {
                        transform.Translate(Vector3.left * xSpeed * Time.deltaTime);
                    }
                    else
                    {
                        waitForElevator = false;
                        state = 2;
                    }
                }
                else
                {
                    if (oneDirection)
                    {
                        // 二楼，去右电梯
                        if (currPos.y < 5f)
                        {
                            diff = currPos.x - minX;
                        }
                        // 其他楼层， 去左电梯
                        else
                        {
                            diff = currPos.x - maxX;
                        }
                    }
                    // 否则就近找电梯
                    else
                    {
                        if (Mathf.Abs(currPos.x - maxX) > Mathf.Abs(currPos.x - minX))
                        {
                            diff = currPos.x - minX;
                        }
                        else
                        {
                            diff = currPos.x - maxX;
                        }
                    }
                    TransferMove(0, diff, 1);
                }
                break;
            case 2: // 跟随电梯运行
                if (Mathf.Abs(currPos.y - (targetFloor - 1) * 3.0f - 0.1f) > 0.5f)
                {
                    GetComponent<Transform>().position = new Vector3(currPos.x, (currElevatorLeft ? left_elevator : right_elevator).GetComponent<Transform>().position.y + 0.15f, currPos.z);
                }
                else if ((currElevatorLeft ? left_elevator : right_elevator).GetComponent<elevatorBehavior>().state == 0)
                {
                    if (currElevatorLeft)
                    {
                        //if (currPos.x > maxX)
                        //{
                        //    transform.Translate(Vector3.left * xSpeed * Time.deltaTime);
                        //}
                        //else
                        //{
                        transform.position = new Vector3(currPos.x, (targetFloor - 1) * 3.0f, currPos.z);
                        state = 3;
                        //}
                    }
                    else
                    {
                        //if (currPos.x < minX)
                        //{
                        //    transform.Translate(Vector3.right * xSpeed * Time.deltaTime);
                        //}
                        //else
                        //{
                        transform.position = new Vector3(currPos.x, (targetFloor - 1) * 3.0f, currPos.z);
                        state = 3;
                        //}
                    }
                }
                break;
            case 3:// 移动x到目标位置
                int armState = machineArm.GetComponent<machineArmBehavior>().state;
                float armX = machineArm.GetComponent<Transform>().position.x;
                float myFloor = currPos.y / 3.0f + 1;

                if (armState == 0 || (armState != 12 && armState != 22 && armState != 13 && armState != 160 && armState != 17 && armState != 27 && armState != 18))
                {
                    // 判断运行楼层,5层以上和2层不与机械手上部分冲突
                    if (myFloor >= 4.5 || myFloor <= 2.5 || Mathf.Abs(currPos.x - armX) > 2.5f)
                    {
                        //Debug.Log(myFloor);
                        float target = (targetX - 1) * 35.0f / 14.0f - 6.3f;
                        if (targetX > 2 && targetX < 14)
                        {
                            diff = currPos.x - target;
                            TransferMove(0, diff, 4);
                            //Debug.Log(diff);
                        }
                        //等电梯
                        else if (targetX <= 2 && right_elevator.GetComponent<elevatorBehavior>().state == 0 && Mathf.Abs(right_elevator.GetComponent<Transform>().position.y - currPos.y) < 0.25f)
                        {
                            diff = currPos.x - target;
                            TransferMove(0, diff, 4);
                        }
                        else if (targetX >= 14 && left_elevator.GetComponent<elevatorBehavior>().state == 0 && Mathf.Abs(left_elevator.GetComponent<Transform>().position.y - currPos.y) < 0.25f)
                        {
                            diff = currPos.x - target;
                            TransferMove(0, diff, 4);
                        }
                    }
                }
                break;
            case 4:// 等待指令，送车或者取车
                armX = machineArm.GetComponent<Transform>().position.x;
                switch (cmd)
                {
                    case 1:// 停车接车，机械手控制
                        if (load == null)
                        {
                            load = machineArm.GetComponent<machineArmBehavior>().myCar;
                            machineArm.GetComponent<machineArmBehavior>().myCar = null;
                        }
                        if (load != null && targetFloor == 2)
                        {
                            load.GetComponent<Transform>().position = currPos + Vector3.up * 0.15f;
                            cmd = 0;
                            state = 0;
                            targetFloor = -1;
                            targetX = -1;
                        }
                        break;
                    case 2:// 停车送车进，controller控制
                        if (load != null)
                        {
                            diff = currPos.z - spaceZ;
                            TransferMove(2, diff, 5);
                        }
                        break;
                    case 3:// 取车进接车，controller控制
                        if (load == null)
                        {
                            diff = currPos.z - spaceZ;
                            TransferMove(2, diff, 5);
                        }
                        break;
                    case 4:// 等机械手接车，controller控制
                        if (load != null)
                        {
                            machineArmBehavior mb = machineArm.GetComponent<machineArmBehavior>();
                            Vector3 armPos = machineArm.GetComponent<Transform>().position;
                            if (mb.state == 160 && Mathf.Abs(armPos.x - currPos.x) < 1.5f && Mathf.Abs(armPos.z - currPos.z) < 0.25f)
                            {
                                if (mb.myCar == null)
                                {
                                    mb.myCar = load;
                                    load = null;
                                    cmd = 0;
                                    state = 0;
                                    targetFloor = -1;
                                    targetX = -1;
                                }
                            }
                        }
                        break;
                    case -1:// 其他情况，复位
                        state = 0;
                        cmd = 0;
                        targetFloor = -1;
                        targetX = -1;
                        break;
                    default: break;
                }
                break;
            case 5:// 从车位移动出来,清除命令
                diff = currPos.z - mainZ;
                //Debug.Log(diff);
                if (cmd == 2)
                {
                    // 加入车库
                    //Debug.Log(load);
                    controller.GetComponent<controller>().AddToGarage(load, targetFloor, targetX);
                    load = null;
                }
                else if (cmd == 3)
                {
                    load = controller.GetComponent<controller>().GetCar(targetFloor, targetX);
                }
                //else
                //{
                //    Debug.Log("中跑车状态跳转出现异常");
                //    return;
                //}
                cmd = 0;
                targetFloor = -1;
                targetX = -1;
                TransferMove(2, diff, 0);
                break;
            default: break;
        }
    }
}
